CREATE TABLE MyNodeInfo (
    my_node_num INTEGER PRIMARY KEY,
    reboot_count INTEGER NOT NULL,
    min_app_version INTEGER NOT NULL
);

-- Stores the latest node info for each node.
CREATE TABLE Node (
    -- Stuff that's only stored here (ephemeral)
    num INTEGER PRIMARY KEY,    -- Unique number (mac addr?) for each node. Primary Key
    user_id TEXT,               -- the typical !c6744jd whatever code (doesn't always show up for bad signal nodes)
    -- Latest NodeHardwareHistory
    user_lname TEXT,            -- User's Long name
    user_sname TEXT,            -- User's Short name
    hw_model INTEGER,           -- What hardware the node is using
    is_licensed INTEGER,        -- is the node owned by a HAM?
    role INTEGER,                  -- What role the node is
    battery_level INTEGER,      -- Battery level, if it has one
    voltage REAL,               -- Voltage from the power supply
    -- Latest NodeGPSHistory
    latitude_i INTEGER,         -- multiply by 1e-7 to get degrees in float.
    longitude_i INTEGER,        -- multiply by 1e-7 to get degrees in float.
    altitude INTEGER,
    location_source INTEGER,       -- Where does the location come from?
    gps_timestamp INTEGER,      -- What time does the gps say it is?
    altitude_source INTEGER,       -- Where does the altitude come from?
    pdop INTEGER,               -- Horizontal, Vertical, and Position Dilution of precision, in 1/100 units.
    hdop INTEGER,
    vdop INTEGER,
    gps_accuracy INTEGER,       -- hardware constant in mm, used to calculate true accuracy (default to about 3m)
    ground_speed INTEGER,       -- in m/s
    ground_track INTEGER,       -- direction of motion
    fix_quality INTEGER,        -- GPS Fix (something called NMEA GxGSA Statement)
    fix_type INTEGER,           -- fix type, whatever that means
    sats_in_view INTEGER,       -- How many satellites we can see
    sensor_id INTEGER,          -- Just incase there's multiple gps devices on the node
    next_update INTEGER,        -- How long until we ask for GPS (in seconds)
    seq_number INTEGER,         -- Sequence number
    precision_bits INTEGER,     -- How many bits of GPS precision is broadcast
    -- Latest NodeRadioHistory
    snr REAL,                   -- Signal to Noise ratio of this message, measured by receiver
    last_heard INTEGER,         -- Last time we heard the message
    channel_utilization REAL,   -- Utilization for the current channel, including well formed TX, RX and malformed RX (aka noise).
    air_util_tx REAL,           -- Percent of airtime for transmission used within the last hour.
    channel INTEGER,            -- The local channel we heard the node on.
    via_mqtt INTEGER,           -- Did we see this node over MQTT instead of LoRA?
    hops_away INTEGER           -- Number of hops away from us in the network. (0 is direct)
);

-- This table stores the historical GPS Location for all seen nodes.
CREATE TABLE NodeGPSHistory (
    primary_id INTEGER PRIMARY KEY AUTOINCREMENT, -- Incrementing number for each gps packet
    num INT,                    -- Unique number for each node.
    latitude_i INTEGER,         -- multiply by 1e-7 to get degrees in float.
    longitude_i INTEGER,        -- multiply by 1e-7 to get degrees in float.
    altitude INTEGER,
    location_source TEXT,       -- Where does the location come from?
    gps_timestamp INTEGER,      -- What time does the gps say it is?
    altitude_source TEXT,       -- Where does the altitude come from?
    pdop INTEGER,               -- Horizontal, Vertical, and Position Dilution of precision, in 1/100 units.
    hdop INTEGER,
    vdop INTEGER,
    gps_accuracy INTEGER,       -- hardware constant in mm, used to calculate true accuracy (default to about 3m)
    ground_speed INTEGER,       -- in m/s
    ground_track INTEGER,       -- direction of motion
    fix_quality INTEGER,        -- GPS Fix (something called NMEA GxGSA Statement)
    fix_type INTEGER,           -- fix type, whatever that means
    sats_in_view INTEGER,       -- How many satellites we can see
    sensor_id INTEGER,          -- Just incase there's multiple gps devices on the node
    next_update INTEGER,        -- How long until we ask for GPS (in seconds)
    seq_number INTEGER,         -- Sequence number
    precision_bits INTEGER      -- How accurate the position is
);

-- This table stores the historical radio stats for all seen nodes.
CREATE TABLE NodeRadioHistory (
    primary_id INTEGER PRIMARY KEY AUTOINCREMENT, -- Incrementing number for each radio packet
    num INT,                    -- Unique number for each node.
    snr REAL,                   -- Signal to Noise ratio of this message, measured by receiver
    channel_utilization REAL,   -- Utilization for the current channel, including well formed TX, RX and malformed RX (aka noise).
    air_util_tx REAL,           -- Percent of airtime for transmission used within the last hour.
    channel INTEGER,            -- The local channel we heard the node on.
    via_mqtt INTEGER,           -- Did we see this node over MQTT instead of LoRA?
    hops_away INTEGER           -- Number of hops away from us in the network. (0 is direct)
);

-- This table stores the historical hardware stats for all seen nodes.
CREATE TABLE NodeHardwareHistory (
    primary_id INTEGER PRIMARY KEY AUTOINCREMENT, -- Incrementing number for each hardware packet
    num INT,                    -- Unique number for each node.
    user_lname TEXT,            -- User's Long name
    user_sname TEXT,            -- User's Short name
    hw_model TEXT,              -- What hardware the node is using
    is_licensed INTEGER,        -- is the node owned by a HAM?
    role TEXT,                  -- What role the node is
    battery_level INTEGER,      -- Battery level, if it has one
    voltage REAL                -- Voltage from the power supply
);

-- This table stores the historical sensor stats for all seen nodes.
-- (save sensor packets properly here)
    -- (TODO: figure out what sensor stuff i can save, and how to save it?)
    -- (I think all of the possible sensors are shown, along with a unknown sensor (add a warning?)

CREATE TABLE Channel (
    channel_index INTEGER PRIMARY KEY,  -- Index of the channel
    psk BLOB,                   -- Encryption of the channel
    name TEXT,                  -- <12 bytes channel name.
    id INTEGER,                 -- used to create global unique channel name, (name.id)
    uplink_enabled INTEGER,     -- Are messages on the mesh sent to MQTT via other nodes?
    downlink_enabled INTEGER,   -- Are messages seen on the internet sent over LoRA?
    position_precision INTEGER, -- How many bits of precision are sent in position packets?
    role TEXT                   -- General 'status' of the channel
);

CREATE TABLE DeviceConfig (
    role INTEGER,               -- Node's Role
    serial_enabled INTEGER,     -- Disables the SerialConsole
    debug_log_enabled INTEGER,  -- defaults off after connection
    button_gpio INTEGER,        -- lets you change the buttons
    buzzer_gpio INTEGER,        -- pin for the buzzer
    rebroadcast_mode INTEGER,   -- role of the node
    node_info_broadcast_secs INTEGER,
    double_tap_as_button_press INTEGER,
    is_managed INTEGER,         -- if set, harder to config
    disable_triple_click INTEGER
);
CREATE TABLE PositionConfig (
    position_broadcast_secs INTEGER,
    position_broadcast_smart_enabled INTEGER,
    fixed_position INTEGER,
    gps_enabled INTEGER,
    gps_update_interval INTEGER,
    gps_attempt_time INTEGER,
    position_flags INTEGER,
    rx_gpio INTEGER,
    tx_gpio INTEGER,
    broadcast_smart_minimum_distance INTEGER,
    broadcast_smart_minimum_interval_secs INTEGER,
    gps_en_gpio INTEGER,
    gps_mode INTEGER
);
CREATE TABLE PowerConfig (
    is_power_saving INTEGER,
    on_battery_shutdown_after_secs INTEGER,
    adc_multiplier_override REAL,
    wait_bluetooth_secs INTEGER,
    sds_secs INTEGER,
    ls_secs INTEGER,
    min_wake_secs INTEGER,
    device_battery_ina_address INTEGER
);
CREATE TABLE NetworkConfig (
    wifi_enabled INTEGER,
    wifi_ssid TEXT,
    wifi_psk TEXT,
    ntp_server TEXT,
    eth_enabled INTEGER,
    address_mode INTEGER,
    ip INTEGER,
    gateway INTEGER,
    subnet INTEGER,
    dns INTEGER,
    rsyslog_server TEXT
);
CREATE TABLE DisplayConfig (
    screen_on_secs INTEGER,
    gps_format INTEGER,
    auto_screen_carousel_secs INTEGER,
    compass_north_top INTEGER,
    flip_screen INTEGER,
    units INTEGER,
    oled INTEGER,
    displaymode INTEGER,
    heading_bold INTEGER,
    wake_on_tap_or_motion INTEGER
);
CREATE TABLE LoRaConfig (
    use_preset INTEGER,
    modem_preset INTEGER,
    bandwidth INTEGER,
    spread_factor INTEGER,
    coding_rate INTEGER,
    frequency_offset REAL,
    region INTEGER,
    hop_limit INTEGER,
    tx_enabled INTEGER,
    tx_power INTEGER,
    channel_num INTEGER,
    override_duty_cycle INTEGER,
    sx126x_rx_boosted_gain INTEGER,
    override_frequency REAL,
    ignore_incoming BLOB,
    ignore_mqtt INTEGER
);
CREATE TABLE BluetoothConfig (
    enabled INTEGER,
    mode INTEGER,
    fixed_pin INTEGER
);

-- Module Configs --

-- MQTT
CREATE TABLE MqttConfig (
    enabled INTEGER,
    address TEXT,
    username TEXT,
    password TEXT,
    encryption_enabled INTEGER,
    json_enabled INTEGER,
    tls_enabled INTEGER,
    root TEXT,
    proxy_to_client_enabled INTEGER,
    map_reporting_enabled INTEGER,
    publish_interval_secs INTEGER,
    position_precision INTEGER
);

-- Serial
CREATE TABLE SerialConfig (
    enabled INTEGER,
    echo INTEGER,
    rxd INTEGER,
    txd INTEGER,
    baud INTEGER,
    timeout INTEGER,
    mode INTEGER,
    override_console_serial_port INTEGER
);

-- External Notification
CREATE TABLE ExternalNotificationConfig (
    enabled INTEGER,
    output_ms INTEGER,
    output INTEGER,
    output_vibra INTEGER,
    output_buzzer INTEGER,
    active INTEGER,
    alert_message INTEGER,
    alert_message_vibra INTEGER,
    alert_message_buzzer INTEGER,
    alert_bell INTEGER,
    alert_bell_vibra INTEGER,
    alert_bell_buzzer INTEGER,
    use_pwm INTEGER,
    nag_timeout INTEGER,
    use_i2s_as_buzzer INTEGER
);

-- Store Forward
CREATE TABLE StoreForwardConfig (
    enabled INTEGER,
    heartbeat INTEGER,
    records INTEGER,
    history_return_max INTEGER,
    history_return_window INTEGER
);

-- Range Test
CREATE TABLE RangeTestConfig (
    enabled INTEGER,
    sender INTEGER,
    save INTEGER
);

-- Telemetry
CREATE TABLE TelemetryConfig (
    device_update_interval INTEGER,
    environment_update_interval INTEGER,
    environment_measurement_enabled INTEGER,
    environment_screen_enabled INTEGER,
    environment_display_fahrenheit INTEGER,
    air_quality_enabled INTEGER,
    air_quality_interval INTEGER,
    power_measurement_enabled INTEGER,
    power_update_interval INTEGER,
    power_screen_enabled INTEGER
);

-- Canned Message
CREATE TABLE CannedMessageConfig (
    rotary1_enabled INTEGER,
    inputbroker_pin_a INTEGER,
    inputbroker_pin_b INTEGER,
    inputbroker_pin_press INTEGER,
    inputbroker_event_cw INTEGER,
    inputbroker_event_ccw INTEGER,
    inputbroker_event_press INTEGER,
    updown1_enabled INTEGER,
    enabled INTEGER,
    allow_input_source TEXT,
    send_bell INTEGER
);

-- Audio
CREATE TABLE AudioConfig (
    codec2_enabled INTEGER,
    ptt_pin INTEGER,
    bitrate INTEGER,
    i2s_ws INTEGER,
    i2s_sd INTEGER,
    i2s_din INTEGER,
    i2s_sck INTEGER
);

-- Remote Hardware
CREATE TABLE RemoteHardwareConfig (
    enabled INTEGER,
    allow_undefined_pin_access INTEGER,
    available_pins BLOB -- Use BLOB to store complex structures like a vector of structs
);

-- NeighborInfo
CREATE TABLE NeighborInfoConfig (
    enabled INTEGER,
    update_interval INTEGER
);

-- AmbientLighting
CREATE TABLE AmbientLightingConfig (
    led_state INTEGER,
    led_current INTEGER,
    red INTEGER,
    green INTEGER,
    blue INTEGER
);

-- Detection Sensor (any gpio pin alarm)
CREATE TABLE DetectionSensorConfig (
    enabled INTEGER,
    minimum_broadcast_secs INTEGER,
    state_broadcast_secs INTEGER,
    send_bell INTEGER,
    name TEXT,
    monitor_pin INTEGER,
    detection_triggered_high INTEGER,
    use_pullup INTEGER
);

-- Pax counter
CREATE TABLE PaxcounterConfig (
    enabled INTEGER,
    paxcounter_update_interval INTEGER
);

-- MeshPacket tables
-- This table stores every single MeshPacket sent over the radio.
-- Since this can contain a Payload_variant, we need to handle all of the following types
-- https://docs.rs/meshtastic/latest/meshtastic/protobufs/enum.PortNum.html
CREATE TABLE MeshPacket (
    primary_id INTEGER PRIMARY KEY AUTOINCREMENT,   -- internal db packet number
    packet_from INTEGER,                            -- sending node number
    packet_to INTEGER,                              -- the (immediate) destination
    channel INTEGER,                                -- what channel was this sent on, however it's a device specific set
    id INTEGER,                                     -- 'unique' id for the packet, only unique per sender, and for ~10 mins
    rx_time INTEGER,                                -- unixtime packet was received, not sent over radio
    rx_snr INTEGER,                                 -- radio quality @ time of rx, not sent over radio
    hop_limt INTEGER,                               -- n hops allowed, 0 is direct neighbours only
    want_ack INTEGER,                               -- does the sender want ack on arrival of destination?
    priority INTEGER,                               -- how important it is in the network
    rx_rssi INTEGER,                                -- rssi of packet, not sent over radio
    via_mqtt INTEGER,                               -- did it pass over mqtt at any point?
    hop_start INTEGER,                              -- what was the hop limit at tx?
    pv_status INTEGER,                              -- NULL = No payload, 1 = decoded, 2 = encrypted
    pv_portnum INTEGER,                             -- what type of data we're sending
    pv_data BLOB,                                   -- the data we're sending.
    pv_want_response INTEGER,                       -- ask the reipient to respond
    pv_dest INTEGER,                                -- the addr of the destination
    pv_source INTEGER,                              -- the original sender node, only used for reliable multihop pkt
    pv_request_id INTEGER,                          -- used for routing or responses, original msg id
    pv_reply_id INTEGER,                            -- is the payload a reply to a previous message?
    pv_emoji INTEGER                                -- is the payload an emoji?
);