// List of widgets available.

use std::fmt;
pub mod about;
pub mod node_list;
pub mod node_info;
pub mod logging;

#[derive(Clone)]
pub(crate) enum WidgetType {
    About,                      // Shows info relevant about YAMS itself.
    NodeList,                   // Shows nodes in a table
    NodeInfo,                   // Shows information about a specific node
    Log,                        // Shows log output from yams (or serial)
    MapView,                    // Shows Nodes on a map
    Settings,                   // Shows Settings
    SignalStrengthHistorical,   // Shows the (average) signal strength of all messages (dropdown to select specific nodes? - will that work with non-direct nodes?)
}

/// Nice formatting for them, for rendering on the tab headers
impl fmt::Display for WidgetType {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            WidgetType::About => write!(f, "About YAMS"),
            WidgetType::NodeList => write!(f, "Node List"),
            WidgetType::Log => write!(f, "Log"),
            WidgetType::MapView => write!(f, "Map View"),
            WidgetType::Settings => write!(f, "Settings"),
            WidgetType::SignalStrengthHistorical => write!(f, "Signal Strength Historical"),
            WidgetType::NodeInfo => write!(f, "Node"),
        }
    }
}

pub fn render_widget(widget_type: &WidgetType, ui: &mut egui::Ui) {
    match widget_type {
        WidgetType::About => {
            about::about_ui(ui);
        },
        WidgetType::NodeList => {
            node_list::nodelist_ui(ui);
        },
        WidgetType::NodeInfo => {
            node_info::nodeinfo_ui(ui);
        },
        WidgetType::Log => {
            logging::log_ui(ui);
        },
        WidgetType::MapView => todo!(),
        WidgetType::Settings => todo!(),
        WidgetType::SignalStrengthHistorical => todo!(),
    }
}